/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache license, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the license for the specific language governing permissions and
 * limitations under the license.
 */
package org.apache.logging.log4j.core.async.perftest;

import com.lmax.disruptor.collections.Histogram;

public interface IPerfTestRunner {
    static final String LINE100 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890!\"#$%&'()-=^~|\\@`[]{};:+*,.<>/?_123456";
    static final String THROUGHPUT_MSG = LINE100 + LINE100 + LINE100 + LINE100
            + LINE100;
    static final String LATENCY_MSG = "Short msg";

    void runThroughputTest(int lines, Histogram histogram);

    void runLatencyTest(int samples, Histogram histogram, long nanoTimeCost,
            int threadCount);
    void shutdown();
    void log(String finalMessage);
}
